# Fracture mechanics module #

## Working with docker containers

Pull mofem image form docker hub,
```
docker pull likask/mofem_fracture_mechanics:latest
```

Fracture docker image contains volume **mofem_build** where all compiled data
are stored. Volume **mofem_build** can be shared between containers.

First create volume in **mofem_build** volume in root virtual machine directory is used to store data independently
form container:
```
docker run --name mofem_fracture_volume likask/mofem_fracture_mechanics:latest /bin/bash
```

Run docker
```
docker run --rm=true -it \
--volumes-from mofem_fracture_volume -v $HOME:$HOME -e HOSTHOME=$HOME \
likask/mofem_fracture_mechanics /bin/bash
```


### Compiling and testing MoFEM modules

```
cd /mofem_build/um
make -j4
ctest -D Experimental
```

### Updating module

```
docker pull likask/mofem_fracture_mechanics:latest
```

Now you can run new container,
```
docker run --rm=true -it \
--volumes-from mofem_fracture_volume -v $HOME:$HOME -e HOSTHOME=$HOME \
likask/mofem_fracture_mechanics /bin/bash
```

You have to recompile code
```
cd /mofem_build/um
make -j4
ctest -D Experimental
```

Note that you all data previously created are still accessed form volume
**mofem_fracture_volume**. You can as well create new volume and start everything
form scratch. For Example new volume **my_new_mofem_fracture_volume** can be created
```
docker run --name my_new_mofem_fracture_volume likask/mofem_fracture_mechanics:latest /bin/bash
```
and you can run container with it
```
docker run --rm=true -it \
--volumes-from my_new_mofem_fracture_volume -v $HOME:$HOME -e HOSTHOME=$HOME \
likask/mofem_fracture_mechanics /bin/bash
```

### Making commitments to **mofem_fracture_mechanics** image

Each time **docker** container is created only changes created in **mofem_fracture_volume** are
permanent. If new packages are installed or any changes are made in code source are lost with
the exit from **docker** container. To make changes permanent two options are possible,
* Create *named* container and use it each time to do work
* Commit changes from *named* container to **mofem_fracture_volume** image.

#### Creating named container

To create named **my_work_conatiner** container
```
docker run -it \
--name my_work_conatiner \
--volumes-from mofem_fracture_volume -v $HOME:$HOME -e HOSTHOME=$HOME \
likask/mofem_fracture_mechanics /bin/bash
```

If you exit from **my_work_conatiner** and execute command
```
docker ps
```
you can see
```
CONTAINER ID        IMAGE                                    COMMAND             CREATED             STATUS                     PORTS               NAMES
398a63ede4e3        likask/mofem_fracture_mechanics          "/bin/bash"         7 seconds ago       Exited (0) 3 seconds ago                       my_work_conatiner
```

You can return to this container again by
```
docker start -i my_work_conatiner
```

#### Committing changes to image

Any changes made to **my_work_conatiner** can be committed to **mofem_fracture_mechanics** and be
permanent. To commit changes
```
docker commit my_work_conatiner my_fracture_mechanics
```
and now you can start container using **my_fracture_mechanics** image
```
docker run -it \
--name my_work_conatiner \
--volumes-from mofem_fracture_volume -v $HOME:$HOME -e HOSTHOME=$HOME \
my_fracture_mechanics /bin/bash
```

Note that you will be still using **mofem_fracture_volume** volume, so changes
made with different containers all still there.

## Check before run

* Check the location of your mesh file
* Check what version of openmpi (or other MPI library) you are using


## Analytical boundary conditions and med format

Note: Example for Docker

* Editing file with analytical boundary conditions
```
jed AnalyticalFun.hpp
```

* Compile changes
```
make
```

* Read med file and setup boundary conditions
```
../tools/read_med -med_file CubeWithCrack.med -meshsets_config make_blocks.config
```

* Run calculations
```
./convergence_study.sh -f out.h5m -l 1e-4 -o 2 -r 1 -d TEST_SIF -g 1e-6 -p 4 --pc_factor_mat_solver_package=mumps
```

* Convert file to VTK format
```
mbconvert TEST_SIF/out_stresses_2_1.h5m TEST_SIF/out_stresses_2_1.vtk
```

* Copy file to Desktop
```
cp TEST_SIF/out_stresses_2_1.vtk $HOSTHOME/Desktop
```

## New development

The code is currently reimplemented,  the structure is modified such that future
changes and improvements will be simpler. This version of the code is better
parallelised and supplemented with multi-grid and field split preconditioners.
Obsolete techniques/data structures are eliminated.


The code is currently reimplemented,  the structure is modified such that future changes and improvements will be simpler. This version of the code is better parallelised and supplemented with multi-grid and field split preconditioners. Obsolete techniques/data structures are eliminated.

Current capabilities:
* Calculation of elastic release energy for linear and **nonlinear elasticity**
* Base functions include singularity of arbitrary order of approximation based on degenerated jacobian by bubble functions in material configuration (quatter point element is based on similar idea)
* Reading residual stresses from arbitrary mesh and use it to calulate stress release energy
* Added expilicit derviative enabling calulation of crack realease enenergy for heterogenous and nonlinear materials
* Added material model to assess fracture toughnes in bone

### Examples of command lines

```
mpirun -np 4 ./crack_propagation \
-my_file crack1.h5m  \
-snes_monitor -snes_atol 1e-8 -snes_rtol 0 \
-snes_converged_reason -snes_linesearch_type bt -snes_linesearch_monitor -ksp_max_it 500 -snes_max_linear_solve_fail 100 -snes_max_fail 100 \
-ksp_monitor -ksp_type fgmres -my_max_post_proc_ref_level 1 -ksp_atol 1e-12 -ksp_rtol 0 \
-my_order 2 \
-mofem_mg_verbose 1 -mofem_mg_coarse_order 1 -mofem_mg_levels 2 \
-mg_coarse_ksp_type preonly -mg_coarse_pc_type lu -mg_coarse_pc_factor_mat_solver_package mumps \
-pc_mg_smoothup 10 -pc_mg_smoothdown 10 -pc_mg_galerkin true -pc_mg_type multiplicative \
-pc_type lu -pc_factor_mat_solver_package mumps \
-my_ref 1 -my_geom_order 1 -my_ref_order 1 -material HOOKE \
-nb_load_steps 4 -load_scale 0.025  2>&1 | tee log0
```

### Converting analytical files


```
../tools/read_med -med_file ./examples/analytical_bc/Mesh_tetra_cube_10.med -meshsets_config ./examples/analytical_bc/make_blocks.config
```

### Running analytical test

```
mpirun -np 4 ./crack_propagation \
-my_file examples/analytical_bc/out_10.h5m  \
-snes_monitor \
-ksp_monitor -ksp_type gmres \
-my_max_post_proc_ref_level 0 -ksp_atol 1e-12 -ksp_rtol 0 \
-is_conservative_force false \
-my_order 2 \
-my_ref 0 -my_geom_order 2 -my_ref_order 2 -material HOOKE \
-mofem_mg_verbose 1 -mofem_mg_coarse_order 1 -mofem_mg_levels 2 \
-mg_coarse_ksp_type preonly -mg_coarse_pc_type lu -mg_coarse_pc_factor_mat_solver_package mumps \
-pc_mg_smoothup 10 -pc_mg_smoothdown 10 -pc_mg_galerkin true -pc_mg_type multiplicative  \
-pc_type lu -pc_factor_mat_solver_package mumps 2>&1 | tee log
```

```
mpirun -np 4 ./crack_propagation \
-my_med_file ./examples/analytical_bc/Mesh_tetra_cube_10.med \
-meshsets_config ./examples/analytical_bc/make_blocks.config \
-snes_monitor \
-ksp_monitor -ksp_type gmres \
-my_max_post_proc_ref_level 0 -ksp_atol 1e-12 -ksp_rtol 0 \
-is_conservative_force false \
-my_order 2 \
-my_ref 0 -my_geom_order 2 -my_ref_order 2 -material HOOKE \
-mofem_mg_verbose 1 -mofem_mg_coarse_order 1 -mofem_mg_levels 2 \
-mg_coarse_ksp_type preonly -mg_coarse_pc_type lu -mg_coarse_pc_factor_mat_solver_package mumps \
-pc_mg_smoothup 10 -pc_mg_smoothdown 10 -pc_mg_galerkin true -pc_mg_type multiplicative  \
-pc_type lu -pc_factor_mat_solver_package mumps 2>&1 | tee log
```

### Running code with internal stresses


Read internal stresses form med file, results are stored in out.h5m:
```
../tools/read_med -med_file examples/stresses_ELGA.rmed
```

Run analysis:
```
mpirun -np 4 ./crack_propagation -my_file ./examples/brick_slice.cub  \
-snes_monitor -ksp_monitor -ksp_type fgmres -my_max_post_proc_ref_level 1 -ksp_atol 1e-12 -ksp_rtol 0 \
-my_order 2 -mofem_mg_verbose 1 -mofem_mg_coarse_order 1 -mofem_mg_levels 2 \
-mg_coarse_ksp_type preonly -mg_coarse_pc_type lu -mg_coarse_pc_factor_mat_solver_package mumps \
-pc_mg_smoothup 10 -pc_mg_smoothdown 10 -pc_mg_galerkin true -pc_mg_type multiplicative \
-pc_type lu -pc_factor_mat_solver_package mumps \
-my_ref 0 -my_geom_order 1 -my_ref_order 0 \
-my_residual_stress_block 1 -my_mwls_approx_file out.h5m
```
